@@ -17,7 +17,11 @@
 
 //! Nom parsers for DNS.
 
-use nom::{IResult, be_u8, be_u16, be_u32};
+use nom::IResult;
+use nom::combinator::rest;
+use nom::error::ErrorKind;
+use nom::multi::length_data;
+use nom::number::streaming::{be_u8, be_u16, be_u32};
 use nom;
 use crate::dns::dns::*;
 
@@ -67,7 +71,7 @@ pub fn dns_parse_name<'a, 'b>(start: &'b [u8],
             pos = &pos[1..];
             break;
         } else if len & 0b1100_0000 == 0 {
-            match length_bytes!(pos, be_u8) {
+            match length_data(be_u8)(pos) as IResult<&[u8],_> {
                 Ok((rem, label)) => {
                     if name.len() > 0 {
                         name.push('.' as u8);
@@ -77,37 +81,37 @@ pub fn dns_parse_name<'a, 'b>(start: &'b [u8],
                 }
                 _ => {
                     return Err(nom::Err::Error(
-                        error_position!(pos, nom::ErrorKind::OctDigit)));
+                        error_position!(pos, ErrorKind::OctDigit)));
                 }
             }
         } else if len & 0b1100_0000 == 0b1100_0000 {
-            match be_u16(pos) {
+            match be_u16(pos) as IResult<&[u8],_> {
                 Ok((rem, leader)) => {
-                    let offset = leader & 0x3fff;
-                    if offset as usize > message.len() {
+                    let offset = usize::from(leader) & 0x3fff;
+                    if offset > message.len() {
                         return Err(nom::Err::Error(
-                            error_position!(pos, nom::ErrorKind::OctDigit)));
+                            error_position!(pos, ErrorKind::OctDigit)));
                     }
-                    pos = &message[offset as usize..];
+                    pos = &message[offset..];
                     if pivot == start {
                         pivot = rem;
                     }
                 }
                 _ => {
                     return Err(nom::Err::Error(
-                        error_position!(pos, nom::ErrorKind::OctDigit)));
+                        error_position!(pos, ErrorKind::OctDigit)));
                 }
             }
         } else {
             return Err(nom::Err::Error(
-                error_position!(pos, nom::ErrorKind::OctDigit)));
+                error_position!(pos, ErrorKind::OctDigit)));
         }
 
         // Return error if we've looped a certain number of times.
         count += 1;
         if count > 255 {
             return Err(nom::Err::Error(
-                error_position!(pos, nom::ErrorKind::OctDigit)));
+                error_position!(pos, ErrorKind::OctDigit)));
         }
 
     }
@@ -140,8 +144,9 @@ fn dns_parse_answer<'a>(slice: &'a [u8], message: &'a [u8], count: usize)
     let mut input = slice;
 
     for _ in 0..count {
-        match closure!(&'a [u8], do_parse!(
-            name: apply!(dns_parse_name, message) >>
+        match do_parse!(
+            input,
+            name: call!(dns_parse_name, message) >>
                 rrtype: be_u16 >>
                 rrclass: be_u16 >>
                 ttl: be_u32 >>
@@ -154,7 +159,7 @@ fn dns_parse_answer<'a>(slice: &'a [u8], message: &'a [u8], count: usize)
                     ttl,
                     data
                 )
-        ))(input) {
+        ) {
             Ok((rem, val)) => {
                 let name = val.0;
                 let rrtype = val.1;
@@ -176,12 +181,13 @@ fn dns_parse_answer<'a>(slice: &'a [u8], message: &'a [u8], count: usize)
                         1
                     }
                 };
-                let result: IResult<&'a [u8], Vec<Vec<u8>>> =
-                    closure!(&'a [u8], do_parse!(
+                let result: IResult<&'a [u8], Vec<DNSRData>> =
+                    do_parse!(
+                        data,
                         rdata: many_m_n!(1, n,
-                                         complete!(apply!(dns_parse_rdata, message, rrtype)))
+                                         complete!(call!(dns_parse_rdata, message, rrtype)))
                             >> (rdata)
-                    ))(data);
+                    );
                 match result {
                     Ok((_, rdatas)) => {
                         for rdata in rdatas {
@@ -209,13 +215,14 @@ fn dns_parse_answer<'a>(slice: &'a [u8], message: &'a [u8], count: usize)
 /// Parse a DNS response.
 pub fn dns_parse_response<'a>(slice: &'a [u8])
                               -> IResult<&[u8], DNSResponse> {
-    let response = closure!(&'a [u8], do_parse!(
+    do_parse!(
+        slice,
         header: dns_parse_header
             >> queries: count!(
-                apply!(dns_parse_query, slice), header.questions as usize)
-            >> answers: apply!(
+                call!(dns_parse_query, slice), header.questions as usize)
+            >> answers: call!(
                 dns_parse_answer, slice, header.answer_rr as usize)
-            >> authorities: apply!(
+            >> authorities: call!(
                 dns_parse_answer, slice, header.authority_rr as usize)
             >> (
                 DNSResponse{
@@ -225,21 +232,20 @@ pub fn dns_parse_response<'a>(slice: &'a [u8])
                     authorities: authorities,
                 }
             )
-    ))(slice);
-
-    return response;
+    )
 }
 
 /// Parse a single DNS query.
 ///
-/// Arguments are suitable for using with apply!:
+/// Arguments are suitable for using with call!:
 ///
-///    apply!(complete_dns_message_buffer)
+///    call!(complete_dns_message_buffer)
 pub fn dns_parse_query<'a>(input: &'a [u8],
                            message: &'a [u8])
                            -> IResult<&'a [u8], DNSQueryEntry> {
-    return closure!(&'a [u8], do_parse!(
-        name: apply!(dns_parse_name, message) >>
+    do_parse!(
+        input,
+        name: call!(dns_parse_name, message) >>
         rrtype: be_u16 >>
         rrclass: be_u16 >>
             (
@@ -249,48 +255,116 @@ pub fn dns_parse_query<'a>(input: &'a [u8],
                     rrclass: rrclass,
                 }
             )
-    ))(input);
+    )
+}
+
+fn dns_parse_rdata_a<'a>(input: &'a [u8]) -> IResult<&'a [u8], DNSRData> {
+    rest(input).map(|(input, data)| (input, DNSRData::A(data.to_vec())))
+}
+
+fn dns_parse_rdata_aaaa<'a>(input: &'a [u8]) -> IResult<&'a [u8], DNSRData> {
+    rest(input).map(|(input, data)| (input, DNSRData::AAAA(data.to_vec())))
+}
+
+fn dns_parse_rdata_cname<'a>(input: &'a [u8], message: &'a [u8])
+                             -> IResult<&'a [u8], DNSRData> {
+    dns_parse_name(input, message).map(|(input, name)|
+            (input, DNSRData::CNAME(name)))
+}
+
+fn dns_parse_rdata_ptr<'a>(input: &'a [u8], message: &'a [u8])
+                           -> IResult<&'a [u8], DNSRData> {
+    dns_parse_name(input, message).map(|(input, name)|
+            (input, DNSRData::PTR(name)))
+}
+
+fn dns_parse_rdata_soa<'a>(input: &'a [u8], message: &'a [u8])
+                           -> IResult<&'a [u8], DNSRData> {
+    do_parse!(
+        input,
+        mname: call!(dns_parse_name, message) >>
+        rname: call!(dns_parse_name, message) >>
+        serial: be_u32 >>
+        refresh: be_u32 >>
+        retry: be_u32 >>
+        expire: be_u32 >>
+        minimum: be_u32 >>
+            (DNSRData::SOA(DNSRDataSOA{
+                mname,
+                rname,
+                serial,
+                refresh,
+                retry,
+                expire,
+                minimum,
+            }))
+    )
+}
+
+fn dns_parse_rdata_mx<'a>(input: &'a [u8], message: &'a [u8])
+                          -> IResult<&'a [u8], DNSRData> {
+    // For MX we skip over the preference field before
+    // parsing out the name.
+    do_parse!(
+        input,
+        be_u16 >>
+        name: call!(dns_parse_name, message) >>
+            (DNSRData::MX(name))
+    )
+}
+
+fn dns_parse_rdata_txt<'a>(input: &'a [u8])
+                           -> IResult<&'a [u8], DNSRData> {
+    do_parse!(
+        input,
+        len: be_u8 >>
+        txt: take!(len) >>
+            (DNSRData::TXT(txt.to_vec()))
+    )
+}
+
+fn dns_parse_rdata_sshfp<'a>(input: &'a [u8])
+                             -> IResult<&'a [u8], DNSRData> {
+    do_parse!(
+        input,
+        algo: be_u8 >>
+        fp_type: be_u8 >>
+        fingerprint: call!(rest) >>
+            (DNSRData::SSHFP(DNSRDataSSHFP{
+                algo,
+                fp_type,
+                fingerprint: fingerprint.to_vec()
+            }))
+    )
+}
+
+fn dns_parse_rdata_unknown<'a>(input: &'a [u8])
+                               -> IResult<&'a [u8], DNSRData> {
+    rest(input).map(|(input, data)| (input, DNSRData::Unknown(data.to_vec())))
 }
 
 pub fn dns_parse_rdata<'a>(input: &'a [u8], message: &'a [u8], rrtype: u16)
-    -> IResult<&'a [u8], Vec<u8>>
+    -> IResult<&'a [u8], DNSRData>
 {
     match rrtype {
-        DNS_RECORD_TYPE_CNAME |
-        DNS_RECORD_TYPE_PTR |
-        DNS_RECORD_TYPE_SOA => {
-            dns_parse_name(input, message)
-        },
-        DNS_RECORD_TYPE_MX => {
-            // For MX we we skip over the preference field before
-            // parsing out the name.
-            closure!(&'a [u8], do_parse!(
-                be_u16 >>
-                name: apply!(dns_parse_name, message) >>
-                    (name)
-            ))(input)
-        },
-        DNS_RECORD_TYPE_TXT => {
-            closure!(&'a [u8], do_parse!(
-                len: be_u8 >>
-                txt: take!(len) >>
-                    (txt.to_vec())
-            ))(input)
-        },
-        _ => {
-            closure!(&'a [u8], do_parse!(
-                data: take!(input.len()) >>
-                    (data.to_vec())
-            ))(input)
-        }
+        DNS_RECORD_TYPE_A => dns_parse_rdata_a(input),
+        DNS_RECORD_TYPE_AAAA => dns_parse_rdata_aaaa(input),
+        DNS_RECORD_TYPE_CNAME => dns_parse_rdata_cname(input, message),
+        DNS_RECORD_TYPE_PTR => dns_parse_rdata_ptr(input, message),
+        DNS_RECORD_TYPE_SOA => dns_parse_rdata_soa(input, message),
+        DNS_RECORD_TYPE_MX => dns_parse_rdata_mx(input, message),
+        DNS_RECORD_TYPE_TXT => dns_parse_rdata_txt(input),
+        DNS_RECORD_TYPE_SSHFP => dns_parse_rdata_sshfp(input),
+        _ => dns_parse_rdata_unknown(input),
     }
 }
 
 /// Parse a DNS request.
 pub fn dns_parse_request<'a>(input: &'a [u8]) -> IResult<&[u8], DNSRequest> {
-    return closure!(&'a [u8], do_parse!(
+    do_parse!(
+        input,
         header: dns_parse_header >>
-        queries: count!(apply!(dns_parse_query, input),
+        queries: count!(call!(dns_parse_query, input),
                         header.questions as usize) >>
             (
                 DNSRequest{
@@ -298,7 +372,7 @@ pub fn dns_parse_request<'a>(input: &'a [u8]) -> IResult<&[u8], DNSRequest> {
                     queries: queries,
                 }
             )
-    ))(input);
+    )
 }
 
 #[cfg(test)]
@@ -503,7 +577,7 @@ mod tests {
                 assert_eq!(answer1.rrclass, 1);
                 assert_eq!(answer1.ttl, 3544);
                 assert_eq!(answer1.data,
-                           "suricata-ids.org".as_bytes().to_vec());
+                           DNSRData::CNAME("suricata-ids.org".as_bytes().to_vec()));
 
                 let answer2 = &response.answers[1];
                 assert_eq!(answer2, &DNSAnswerEntry{
@@ -511,7 +585,7 @@ mod tests {
                     rrtype: 1,
                     rrclass: 1,
                     ttl: 244,
-                    data: [192, 0, 78, 24].to_vec(),
+                    data: DNSRData::A([192, 0, 78, 24].to_vec()),
                 });
 
                 let answer3 = &response.answers[2];
@@ -520,7 +594,7 @@ mod tests {
                     rrtype: 1,
                     rrclass: 1,
                     ttl: 244,
-                    data: [192, 0, 78, 25].to_vec(),
+                    data: DNSRData::A([192, 0, 78, 25].to_vec()),
                 })
 
             },
@@ -530,4 +604,106 @@ mod tests {
         }
     }
 
+    #[test]
+    fn test_dns_parse_response_nxdomain_soa() {
+        // DNS response with an SOA authority record from
+        // dns-udp-nxdomain-soa.pcap.
+        let pkt: &[u8] = &[
+                        0x82, 0x95, 0x81, 0x83, 0x00, 0x01, /* j....... */
+            0x00, 0x00, 0x00, 0x01, 0x00, 0x01, 0x03, 0x64, /* .......d */
+            0x6e, 0x65, 0x04, 0x6f, 0x69, 0x73, 0x66, 0x03, /* ne.oisf. */
+            0x6e, 0x65, 0x74, 0x00, 0x00, 0x01, 0x00, 0x01, /* net..... */
+            0xc0, 0x10, 0x00, 0x06, 0x00, 0x01, 0x00, 0x00, /* ........ */
+            0x03, 0x83, 0x00, 0x45, 0x06, 0x6e, 0x73, 0x2d, /* ...E.ns- */
+            0x31, 0x31, 0x30, 0x09, 0x61, 0x77, 0x73, 0x64, /* 110.awsd */
+            0x6e, 0x73, 0x2d, 0x31, 0x33, 0x03, 0x63, 0x6f, /* ns-13.co */
+            0x6d, 0x00, 0x11, 0x61, 0x77, 0x73, 0x64, 0x6e, /* m..awsdn */
+            0x73, 0x2d, 0x68, 0x6f, 0x73, 0x74, 0x6d, 0x61, /* s-hostma */
+            0x73, 0x74, 0x65, 0x72, 0x06, 0x61, 0x6d, 0x61, /* ster.ama */
+            0x7a, 0x6f, 0x6e, 0xc0, 0x3b, 0x00, 0x00, 0x00, /* zon.;... */
+            0x01, 0x00, 0x00, 0x1c, 0x20, 0x00, 0x00, 0x03, /* .... ... */
+            0x84, 0x00, 0x12, 0x75, 0x00, 0x00, 0x01, 0x51, /* ...u...Q */
+            0x80, 0x00, 0x00, 0x29, 0x02, 0x00, 0x00, 0x00, /* ...).... */
+            0x00, 0x00, 0x00, 0x00                          /* .... */
+        ];
+
+        let res = dns_parse_response(pkt);
+        match res {
+            Ok((rem, response)) => {
+
+                // For now we have some remainder data as there is an
+                // additional record type we don't parse yet.
+                assert!(rem.len() > 0);
+
+                assert_eq!(response.header, DNSHeader{
+                    tx_id: 0x8295,
+                    flags: 0x8183,
+                    questions: 1,
+                    answer_rr: 0,
+                    authority_rr: 1,
+                    additional_rr: 1,
+                });
+
+                assert_eq!(response.authorities.len(), 1);
+
+                let authority = &response.authorities[0];
+                assert_eq!(authority.name,
+                           "oisf.net".as_bytes().to_vec());
+                assert_eq!(authority.rrtype, 6);
+                assert_eq!(authority.rrclass, 1);
+                assert_eq!(authority.ttl, 899);
+                assert_eq!(authority.data,
+                           DNSRData::SOA(DNSRDataSOA{
+                               mname: "ns-110.awsdns-13.com".as_bytes().to_vec(),
+                               rname: "awsdns-hostmaster.amazon.com".as_bytes().to_vec(),
+                               serial: 1,
+                               refresh: 7200,
+                               retry: 900,
+                               expire: 1209600,
+                               minimum: 86400,
+                           }));
+            },
+            _ => {
+                assert!(false);
+            }
+        }
+    }
+
+    #[test]
+    fn test_dns_parse_rdata_sshfp() {
+        // Dummy data since we don't have a pcap sample.
+        let data: &[u8] = &[
+            // algo: DSS
+            0x02,
+            // fp_type: SHA-1
+            0x01,
+            // fingerprint: 123456789abcdef67890123456789abcdef67890
+            0x12, 0x34, 0x56, 0x78, 0x9a, 0xbc, 0xde, 0xf6, 0x78, 0x90,
+            0x12, 0x34, 0x56, 0x78, 0x9a, 0xbc, 0xde, 0xf6, 0x78, 0x90
+        ];
+
+        let res = dns_parse_rdata_sshfp(data);
+        match res {
+            Ok((rem, rdata)) => {
+
+                // The data should be fully parsed.
+                assert_eq!(rem.len(), 0);
+
+                match rdata {
+                    DNSRData::SSHFP(sshfp) => {
+                        assert_eq!(sshfp.algo, 2);
+                        assert_eq!(sshfp.fp_type, 1);
+                        assert_eq!(sshfp.fingerprint, &data[2..]);
+                    },
+                    _ => {
+                        assert!(false);
+                    }
+                }
+            },
+            _ => {
+                assert!(false);
+            }
+        }
+    }
+
 }
