@@ -281,6 +281,9 @@ impl<'a> SNMPTransaction<'a> {
         if self.events != std::ptr::null_mut() {
             core::sc_app_layer_decoder_events_free_events(&mut self.events);
         }
+        if let Some(de_state) = self.de_state {
+            core::sc_detect_engine_state_free(de_state);
+        }
     }
 }
 
