@@ -27,7 +27,7 @@
 void AppLayerExpectationSetup(void);
 int AppLayerExpectationCreate(Flow *f, int direction, Port src, Port dst,
                               AppProto alproto, void *data);
-AppProto AppLayerExpectationHandle(Flow *f, int direction);
+AppProto AppLayerExpectationHandle(Flow *f, uint8_t flags);
 int AppLayerExpectationGetDataId(void);
 
 void AppLayerExpectationClean(Flow *f);
