@@ -33,8 +33,7 @@
 #include "app-layer-parser.h"
 
 #include "app-layer-ikev2.h"
-
-#include "rust-ikev2-ikev2-gen.h"
+#include "rust.h"
 
 void RegisterIKEV2Parsers(void)
 {
