@@ -1,4 +1,4 @@
-/* Copyright (C) 2017 Open Information Security Foundation
+/* Copyright (C) 2017-2020 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -26,13 +26,11 @@
 
 #include "rust.h"
 #include "app-layer-smb.h"
-#include "rust-smb-smb-gen.h"
-#include "rust-smb-files-gen.h"
 #include "util-misc.h"
 
 #define MIN_REC_SIZE 32+4 // SMB hdr + nbss hdr
 
-static int SMBTCPParseRequest(Flow *f, void *state,
+static AppLayerResult SMBTCPParseRequest(Flow *f, void *state,
         AppLayerParserState *pstate, const uint8_t *input, uint32_t input_len,
         void *local_data, const uint8_t flags)
 {
@@ -40,21 +38,20 @@ static int SMBTCPParseRequest(Flow *f, void *state,
     uint16_t file_flags = FileFlowToFlags(f, STREAM_TOSERVER);
     rs_smb_setfileflags(0, state, file_flags|FILE_USE_DETECT);
 
-    int res;
     if (input == NULL && input_len > 0) {
-        res = rs_smb_parse_request_tcp_gap(state, input_len);
+        AppLayerResult res = rs_smb_parse_request_tcp_gap(state, input_len);
+        SCLogDebug("SMB request GAP of %u bytes, retval %d", input_len, res.status);
+        SCReturnStruct(res);
     } else {
-        res = rs_smb_parse_request_tcp(f, state, pstate, input, input_len,
-            local_data, flags);
-    }
-    if (res != 1) {
+        AppLayerResult res = rs_smb_parse_request_tcp(f, state, pstate,
+                input, input_len, local_data, flags);
         SCLogDebug("SMB request%s of %u bytes, retval %d",
-                (input == NULL && input_len > 0) ? " is GAP" : "", input_len, res);
+                (input == NULL && input_len > 0) ? " is GAP" : "", input_len, res.status);
+        SCReturnStruct(res);
     }
-    return res;
 }
 
-static int SMBTCPParseResponse(Flow *f, void *state,
+static AppLayerResult SMBTCPParseResponse(Flow *f, void *state,
         AppLayerParserState *pstate, const uint8_t *input, uint32_t input_len,
         void *local_data, const uint8_t flags)
 {
@@ -63,18 +60,15 @@ static int SMBTCPParseResponse(Flow *f, void *state,
     rs_smb_setfileflags(1, state, file_flags|FILE_USE_DETECT);
 
     SCLogDebug("SMBTCPParseResponse %p/%u", input, input_len);
-    int res;
     if (input == NULL && input_len > 0) {
-        res = rs_smb_parse_response_tcp_gap(state, input_len);
+        AppLayerResult res = rs_smb_parse_response_tcp_gap(state, input_len);
+        SCLogDebug("SMB response GAP of %u bytes, retval %d", input_len, res.status);
+        SCReturnStruct(res);
     } else {
-        res = rs_smb_parse_response_tcp(f, state, pstate, input, input_len,
-            local_data, flags);
+        AppLayerResult res = rs_smb_parse_response_tcp(f, state, pstate,
+                input, input_len, local_data, flags);
+        SCReturnStruct(res);
     }
-    if (res != 1) {
-        SCLogDebug("SMB response%s of %u bytes, retval %d",
-                (input == NULL && input_len > 0) ? " is GAP" : "", input_len, res);
-    }
-    return res;
 }
 
 static uint16_t SMBTCPProbe(Flow *f, uint8_t direction,
@@ -155,16 +149,6 @@ static AppLayerGetTxIterTuple SMBGetTxIterator(
 }
 
 
-static void SMBSetTxLogged(void *alstate, void *tx, uint32_t logger)
-{
-    rs_smb_tx_set_logged(alstate, tx, logger);
-}
-
-static LoggerId SMBGetTxLogged(void *alstate, void *tx)
-{
-    return rs_smb_tx_get_logged(alstate, tx);
-}
-
 static void SMBStateTransactionFree(void *state, uint64_t tx_id)
 {
     rs_smb_state_tx_free(state, tx_id);
@@ -203,16 +187,6 @@ static int SMBGetEventInfo(const char *event_name, int *event_id,
     return rs_smb_state_get_event_info(event_name, event_id, event_type);
 }
 
-static void SMBSetDetectFlags(void *tx, uint8_t dir, uint64_t flags)
-{
-    rs_smb_tx_set_detect_flags(tx, dir, flags);
-}
-
-static uint64_t SMBGetDetectFlags(void *tx, uint8_t dir)
-{
-    return rs_smb_tx_get_detect_flags(tx, dir);
-}
-
 static void SMBStateTruncate(void *state, uint8_t direction)
 {
     return rs_smb_state_truncate(state, direction);
@@ -316,18 +290,16 @@ void RegisterSMBParsers(void)
         AppLayerParserRegisterGetTxIterator(IPPROTO_TCP, ALPROTO_SMB, SMBGetTxIterator);
         AppLayerParserRegisterGetTxCnt(IPPROTO_TCP, ALPROTO_SMB,
                 SMBGetTxCnt);
-        AppLayerParserRegisterLoggerFuncs(IPPROTO_TCP, ALPROTO_SMB,
-                SMBGetTxLogged, SMBSetTxLogged);
         AppLayerParserRegisterGetStateProgressFunc(IPPROTO_TCP, ALPROTO_SMB,
                 SMBGetAlstateProgress);
         AppLayerParserRegisterGetStateProgressCompletionStatus(ALPROTO_SMB,
                 rs_smb_state_progress_completion_status);
-        AppLayerParserRegisterDetectFlagsFuncs(IPPROTO_TCP, ALPROTO_SMB,
-                                               SMBGetDetectFlags, SMBSetDetectFlags);
         AppLayerParserRegisterTruncateFunc(IPPROTO_TCP, ALPROTO_SMB,
                                           SMBStateTruncate);
         AppLayerParserRegisterGetFilesFunc(IPPROTO_TCP, ALPROTO_SMB, SMBGetFiles);
 
+        AppLayerParserRegisterTxDataFunc(IPPROTO_TCP, ALPROTO_SMB, rs_smb_get_tx_data);
+
         /* This parser accepts gaps. */
         AppLayerParserRegisterOptionFlags(IPPROTO_TCP, ALPROTO_SMB,
                 APP_LAYER_PARSER_OPT_ACCEPT_GAPS);
