@@ -628,7 +628,6 @@ static json_t *RulesGroupPrintSghStats(const SigGroupHead *sgh,
     uint32_t payload_no_mpm_cnt = 0;
     uint32_t syn_cnt = 0;
 
-    uint32_t mpms_total = 0;
     uint32_t mpms_min = 0;
     uint32_t mpms_max = 0;
 
@@ -737,7 +736,6 @@ static json_t *RulesGroupPrintSghStats(const SigGroupHead *sgh,
             }
 
             uint32_t w = PatternStrength(cd->content, cd->content_len);
-            mpms_total += w;
             if (mpms_min == 0)
                 mpms_min = w;
             if (w < mpms_min)
