@@ -46,6 +46,7 @@ enum DetectKeywordId {
     DETECT_ICODE,
     DETECT_ICMP_ID,
     DETECT_ICMP_SEQ,
+    DETECT_ICMPV4HDR,
     DETECT_DSIZE,
 
     DETECT_FLOW,
@@ -71,6 +72,7 @@ enum DetectKeywordId {
     DETECT_RAWBYTES,
     DETECT_BYTETEST,
     DETECT_BYTEJUMP,
+    DETECT_BYTEMATH,
     DETECT_SAMEIP,
     DETECT_GEOIP,
     DETECT_IPPROTO,
@@ -160,6 +162,10 @@ enum DetectKeywordId {
     DETECT_AL_SSH_PROTOVERSION,
     DETECT_AL_SSH_SOFTWARE,
     DETECT_AL_SSH_SOFTWAREVERSION,
+    DETECT_AL_SSH_HASSH,
+    DETECT_AL_SSH_HASSH_SERVER,
+    DETECT_AL_SSH_HASSH_STRING,
+    DETECT_AL_SSH_HASSH_SERVER_STRING,
     DETECT_AL_SSL_VERSION,
     DETECT_AL_SSL_STATE,
     DETECT_BYTE_EXTRACT,
@@ -167,6 +173,15 @@ enum DetectKeywordId {
     DETECT_PKT_DATA,
     DETECT_AL_APP_LAYER_EVENT,
 
+    DETECT_HTTP2_FRAMETYPE,
+    DETECT_HTTP2_ERRORCODE,
+    DETECT_HTTP2_PRIORITY,
+    DETECT_HTTP2_WINDOW,
+    DETECT_HTTP2_SIZEUPDATE,
+    DETECT_HTTP2_SETTINGS,
+    DETECT_HTTP2_HEADERNAME,
+    DETECT_HTTP2_HEADER,
+
     DETECT_DCE_IFACE,
     DETECT_DCE_OPNUM,
     DETECT_DCE_STUB_DATA,
@@ -178,6 +193,8 @@ enum DetectKeywordId {
     DETECT_ENGINE_EVENT,
     DETECT_STREAM_EVENT,
 
+    DETECT_CONFIG,
+
     DETECT_FILENAME,
     DETECT_FILE_NAME,
     DETECT_FILEEXT,
@@ -233,11 +250,15 @@ enum DetectKeywordId {
     DETECT_AL_SIP_STAT_MSG,
     DETECT_AL_SIP_REQUEST_LINE,
     DETECT_AL_SIP_RESPONSE_LINE,
-
+    DETECT_AL_RFB_SECRESULT,
+    DETECT_AL_RFB_SECTYPE,
+    DETECT_AL_RFB_NAME,
     DETECT_TEMPLATE,
     DETECT_TEMPLATE2,
     DETECT_IPV4HDR,
     DETECT_IPV6HDR,
+    DETECT_ICMPV6HDR,
+    DETECT_ICMPV6MTU,
     DETECT_TCPHDR,
     DETECT_UDPHDR,
     DETECT_TCPMSS,
@@ -247,6 +268,22 @@ enum DetectKeywordId {
     DETECT_AL_SNMP_VERSION,
     DETECT_AL_SNMP_COMMUNITY,
     DETECT_AL_SNMP_PDU_TYPE,
+    DETECT_AL_MQTT_TYPE,
+    DETECT_AL_MQTT_FLAGS,
+    DETECT_AL_MQTT_QOS,
+    DETECT_AL_MQTT_PROTOCOL_VERSION,
+    DETECT_AL_MQTT_REASON_CODE,
+    DETECT_AL_MQTT_CONNECT_FLAGS,
+    DETECT_AL_MQTT_CONNECT_CLIENTID,
+    DETECT_AL_MQTT_CONNECT_USERNAME,
+    DETECT_AL_MQTT_CONNECT_PASSWORD,
+    DETECT_AL_MQTT_CONNECT_WILLTOPIC,
+    DETECT_AL_MQTT_CONNECT_WILLMESSAGE,
+    DETECT_AL_MQTT_CONNACK_SESSION_PRESENT,
+    DETECT_AL_MQTT_PUBLISH_TOPIC,
+    DETECT_AL_MQTT_PUBLISH_MESSAGE,
+    DETECT_AL_MQTT_SUBSCRIBE_TOPIC,
+    DETECT_AL_MQTT_UNSUBSCRIBE_TOPIC,
     DETECT_AL_TEMPLATE_BUFFER,
 
     DETECT_BYPASS,
@@ -259,6 +296,8 @@ enum DetectKeywordId {
     DETECT_TRANSFORM_SHA1,
     DETECT_TRANSFORM_SHA256,
     DETECT_TRANSFORM_DOTPREFIX,
+    DETECT_TRANSFORM_PCREXFORM,
+    DETECT_TRANSFORM_URL_DECODE,
 
     /* make sure this stays last */
     DETECT_TBLSIZE,
