@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2019 Open Information Security Foundation
+/* Copyright (C) 2007-2020 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -36,11 +36,12 @@
 
 #define PARSE_REGEX "^\\s*(\\d+|\"\\d+\")\\s*$"
 
-static pcre *regex = NULL;
-static pcre_extra *regex_study = NULL;
+static DetectParseRegex parse_regex;
 
 static int DetectPrioritySetup (DetectEngineCtx *, Signature *, const char *);
-void SCPriorityRegisterTests(void);
+#ifdef UNITTESTS
+static void PriorityRegisterTests(void);
+#endif
 
 /**
  * \brief Registers the handler functions for the "priority" keyword
@@ -51,20 +52,20 @@ void DetectPriorityRegister (void)
     sigmatch_table[DETECT_PRIORITY].desc = "rules with a higher priority will be examined first";
     sigmatch_table[DETECT_PRIORITY].url = "/rules/meta.html#priority";
     sigmatch_table[DETECT_PRIORITY].Setup = DetectPrioritySetup;
-    sigmatch_table[DETECT_PRIORITY].RegisterTests = SCPriorityRegisterTests;
-
-    DetectSetupParseRegexes(PARSE_REGEX, &regex, &regex_study);
+#ifdef UNITTESTS
+    sigmatch_table[DETECT_PRIORITY].RegisterTests = PriorityRegisterTests;
+#endif
+    DetectSetupParseRegexes(PARSE_REGEX, &parse_regex);
 }
 
 static int DetectPrioritySetup (DetectEngineCtx *de_ctx, Signature *s, const char *rawstr)
 {
     char copy_str[128] = "";
 
-#define MAX_SUBSTRINGS 30
     int ret = 0;
     int ov[MAX_SUBSTRINGS];
 
-    ret = pcre_exec(regex, regex_study, rawstr, strlen(rawstr), 0, 0, ov, 30);
+    ret = DetectParsePcreExec(&parse_regex, rawstr, 0, 0, ov, 30);
     if (ret < 0) {
         SCLogError(SC_ERR_PCRE_MATCH, "Invalid Priority in Signature "
                      "- %s", rawstr);
@@ -165,15 +166,13 @@ static int DetectPriorityTest02(void)
     DetectEngineCtxFree(de_ctx);
     PASS;
 }
-#endif /* UNITTESTS */
 
 /**
  * \brief This function registers unit tests for Classification Config API.
  */
-void SCPriorityRegisterTests(void)
+static void PriorityRegisterTests(void)
 {
-#ifdef UNITTESTS
     UtRegisterTest("DetectPriorityTest01", DetectPriorityTest01);
     UtRegisterTest("DetectPriorityTest02", DetectPriorityTest02);
-#endif /* UNITTESTS */
 }
+#endif /* UNITTESTS */
