@@ -115,7 +115,6 @@ typedef struct FlowTimeoutCounters_ {
     uint32_t rows_checked;
     uint32_t rows_skipped;
     uint32_t rows_empty;
-    uint32_t rows_busy;
     uint32_t rows_maxlen;
 
     uint32_t flows_checked;
@@ -254,55 +253,6 @@ static inline int FlowBypassedTimeout(Flow *f, struct timeval *ts,
     return 1;
 }
 
-/** \internal
- *  \brief See if we can really discard this flow. Check use_cnt reference
- *         counter and force reassembly if necessary.
- *
- *  \param f flow
- *  \param ts timestamp
- *
- *  \retval 0 not timed out just yet
- *  \retval 1 fully timed out, lets kill it
- */
-#if 0
-static inline int FlowManagerFlowTimedOut(Flow *f, struct timeval *ts,
-                                   FlowTimeoutCounters *counters)
-{
-    /* never prune a flow that is used by a packet we
-     * are currently processing in one of the threads */
-    if (f->use_cnt > 0) {
-        return 0;
-    }
-
-    if (!FlowBypassedTimeout(f, ts, counters)) {
-        return 0;
-    }
-
-    int server = 0, client = 0;
-
-    if (!(f->flags & FLOW_TIMEOUT_REASSEMBLY_DONE) &&
-#ifdef CAPTURE_OFFLOAD
-            f->flow_state != FLOW_STATE_CAPTURE_BYPASSED &&
-#endif
-            f->flow_state != FLOW_STATE_LOCAL_BYPASSED &&
-            FlowForceReassemblyNeedReassembly(f, &server, &client) == 1) {
-        FlowForceReassemblyForFlow(f, server, client);
-        return 0;
-    }
-#ifdef DEBUG
-    /* this should not be possible */
-    BUG_ON(f->use_cnt > 0);
-#endif
-
-    return 1;
-}
-#endif
-
-static inline int FMTryLockBucket(FlowBucket *fb)
-{
-    int r = FBLOCK_TRYLOCK(fb);
-    return r;
-}
 static inline void FMFlowLock(Flow *f)
 {
     FLOWLOCK_WRLOCK(f);
@@ -324,18 +274,17 @@ static uint32_t ProcessAsideQueue(FlowManagerTimeoutThread *td, FlowTimeoutCount
     while ((f = FlowQueuePrivateGetFromTop(&td->aside_queue)) != NULL) {
         /* flow is still locked */
 
-        if (f->proto == IPPROTO_TCP &&
-                !(f->flags & FLOW_TIMEOUT_REASSEMBLY_DONE) &&
+        if (f->proto == IPPROTO_TCP && !(f->flags & FLOW_TIMEOUT_REASSEMBLY_DONE) &&
 #ifdef CAPTURE_OFFLOAD
                 f->flow_state != FLOW_STATE_CAPTURE_BYPASSED &&
 #endif
                 f->flow_state != FLOW_STATE_LOCAL_BYPASSED &&
-                FlowForceReassemblyNeedReassembly(f) == 1)
-        {
+                FlowForceReassemblyNeedReassembly(f) == 1) {
+            /* Send the flow to its thread */
             FlowForceReassemblyForFlow(f);
+            FLOWLOCK_UNLOCK(f);
             /* flow ownership is passed to the worker thread */
 
-            /* flow remains locked */
             counters->flows_aside_needs_work++;
             continue;
         }
@@ -392,18 +341,22 @@ static void FlowManagerHashRowTimeout(FlowManagerTimeoutThread *td,
 
         Flow *next_flow = f->next;
 
-        counters->flows_timeout++;
-
         /* never prune a flow that is used by a packet we
          * are currently processing in one of the threads */
         if (f->use_cnt > 0 || !FlowBypassedTimeout(f, ts, counters)) {
             FLOWLOCK_UNLOCK(f);
             prev_f = f;
-            counters->flows_timeout_inuse++;
+            if (f->use_cnt > 0) {
+                counters->flows_timeout_inuse++;
+            }
             f = f->next;
             continue;
         }
 
+        f->flow_end_flags |= FLOW_END_FLAG_TIMEOUT;
+
+        counters->flows_timeout++;
+
         RemoveFromHash(f, prev_f);
 
         FlowQueuePrivateAppendFlow(&td->aside_queue, f);
@@ -426,7 +379,7 @@ static void FlowManagerHashRowClearEvictedList(FlowManagerTimeoutThread *td,
         f->next = NULL;
         f->fb = NULL;
 
-        DEBUG_VALIDATE_BUG_ON(f->use_cnt > 0 || !FlowBypassedTimeout(f, ts, counters));
+        DEBUG_VALIDATE_BUG_ON(f->use_cnt > 0);
 
         FlowQueuePrivateAppendFlow(&td->aside_queue, f);
         /* flow is still locked in the queue */
@@ -454,7 +407,6 @@ static uint32_t FlowTimeoutHash(FlowManagerTimeoutThread *td,
     const int emergency = ((SC_ATOMIC_GET(flow_flags) & FLOW_EMERGENCY));
     const uint32_t rows_checked = hash_max - hash_min;
     uint32_t rows_skipped = 0;
-    uint32_t rows_busy = 0;
     uint32_t rows_empty = 0;
 
 #if __WORDSIZE==64
@@ -478,43 +430,33 @@ static uint32_t FlowTimeoutHash(FlowManagerTimeoutThread *td,
         for (uint32_t i = 0; i < check; i++) {
             FlowBucket *fb = &flow_hash[idx+i];
             if ((check_bits & ((TYPE)1 << (TYPE)i)) != 0 && SC_ATOMIC_GET(fb->next_ts) <= (int32_t)ts->tv_sec) {
-                if (FMTryLockBucket(fb) == 0) {
-                    Flow *evicted = NULL;
-                    if (fb->evicted != NULL || fb->head != NULL) {
-                        /* if evicted is set, we only process that list right now.
-                         * Since its set we've had traffic that touched this row
-                         * very recently, and there is a good chance more of it will
-                         * come in in the near future. So unlock the row asap and leave
-                         * the possible eviction of flows to the packet lookup path. */
-                        if (fb->evicted != NULL) {
-                            /* transfer out of bucket so we can do additional work outside
-                             * of the bucket lock */
-                            evicted = fb->evicted;
-                            fb->evicted = NULL;
-                        } else if (fb->head != NULL) {
-                            int32_t next_ts = 0;
-                            FlowManagerHashRowTimeout(td,
-                                    fb->head, ts, emergency, counters, &next_ts);
-
-                            if (SC_ATOMIC_GET(fb->next_ts) != next_ts)
-                                SC_ATOMIC_SET(fb->next_ts, next_ts);
-                        }
-                        if (fb->evicted == NULL && fb->head == NULL) {
-                            SC_ATOMIC_SET(fb->next_ts, INT_MAX);
-                        } else if (fb->evicted != NULL && fb->head == NULL) {
-                            SC_ATOMIC_SET(fb->next_ts, 0);
-                        }
-                    } else {
-                        SC_ATOMIC_SET(fb->next_ts, INT_MAX);
-                        rows_empty++;
+                FBLOCK_LOCK(fb);
+                Flow *evicted = NULL;
+                if (fb->evicted != NULL || fb->head != NULL) {
+                    if (fb->evicted != NULL) {
+                        /* transfer out of bucket so we can do additional work outside
+                         * of the bucket lock */
+                        evicted = fb->evicted;
+                        fb->evicted = NULL;
                     }
-                    FBLOCK_UNLOCK(fb);
-                    /* processed evicted list */
-                    if (evicted) {
-                        FlowManagerHashRowClearEvictedList(td, evicted, ts, counters);
+                    if (fb->head != NULL) {
+                        int32_t next_ts = 0;
+                        FlowManagerHashRowTimeout(td, fb->head, ts, emergency, counters, &next_ts);
+
+                        if (SC_ATOMIC_GET(fb->next_ts) != next_ts)
+                            SC_ATOMIC_SET(fb->next_ts, next_ts);
+                    }
+                    if (fb->evicted == NULL && fb->head == NULL) {
+                        SC_ATOMIC_SET(fb->next_ts, INT_MAX);
                     }
                 } else {
-                    rows_busy++;
+                    SC_ATOMIC_SET(fb->next_ts, INT_MAX);
+                    rows_empty++;
+                }
+                FBLOCK_UNLOCK(fb);
+                /* processed evicted list */
+                if (evicted) {
+                    FlowManagerHashRowClearEvictedList(td, evicted, ts, counters);
                 }
             } else {
                 rows_skipped++;
@@ -527,7 +469,6 @@ static uint32_t FlowTimeoutHash(FlowManagerTimeoutThread *td,
 
     counters->rows_checked += rows_checked;
     counters->rows_skipped += rows_skipped;
-    counters->rows_busy += rows_busy;
     counters->rows_empty += rows_empty;
 
     if (td->aside_queue.len) {
@@ -548,8 +489,10 @@ static uint32_t FlowTimeoutHashInChunks(FlowManagerTimeoutThread *td,
 
     const uint32_t min = iter * chunk_size + hash_min;
     uint32_t max = min + chunk_size;
+    /* we start at beginning of hash at next iteration so let's check
+     * hash till the end */
     if (iter + 1 == chunks) {
-        max = rows;
+        max = hash_max;
     }
     const uint32_t cnt = FlowTimeoutHash(td, ts, min, max, counters);
     return cnt;
@@ -791,8 +734,10 @@ static TmEcode FlowManager(ThreadVars *th_v, void *thread_data)
 */
     memset(&ts, 0, sizeof(ts));
     uint32_t hash_passes = 0;
+#ifdef FM_PROFILE
     uint32_t hash_row_checks = 0;
     uint32_t hash_passes_chunks = 0;
+#endif
     uint32_t hash_full_passes = 0;
 
     const uint32_t min_timeout = FlowTimeoutsMin();
@@ -881,16 +826,18 @@ static TmEcode FlowManager(ThreadVars *th_v, void *thread_data)
             const uint32_t secs_passed = rt - flow_last_sec;
 
             /* try to time out flows */
-            FlowTimeoutCounters counters = { 0, 0, 0, 0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
+            FlowTimeoutCounters counters = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 
             if (emerg) {
                 /* in emergency mode, do a full pass of the hash table */
                 FlowTimeoutHash(&ftd->timeout, &ts, ftd->min, ftd->max, &counters);
                 hash_passes++;
                 hash_full_passes++;
-                hash_passes_chunks += 1;
                 hash_passes++;
+#ifdef FM_PROFILE
+                hash_passes_chunks += 1;
                 hash_row_checks += counters.rows_checked;
+#endif
                 StatsIncr(th_v, ftd->cnt.flow_mgr_full_pass);
             } else {
                 /* non-emergency mode: scan part of the hash */
@@ -906,8 +853,10 @@ static TmEcode FlowManager(ThreadVars *th_v, void *thread_data)
                     }
                 }
                 hash_passes++;
+#ifdef FM_PROFILE
                 hash_row_checks += counters.rows_checked;
                 hash_passes_chunks += chunks;
+#endif
             }
             flow_last_sec = rt;
 
